/******************************************************
* Copyright (c):   2004, All Rights Reserved.
* Project:         CS 297 Deliverable 1 N-gram Word Predictor
* File:            NgramLearner.cpp
* Purpose:         Responsible for doing the learning.
* Start date:      8/30/2004
* Programmer:      Leo Lee
*
*******************************************************/

#include "stdafx.h"
#include "NgramWordPredictor.h"
#include "NgramLearner.h"
#include "NgramNode.h"
#include "NgramWordPredictorDoc.h"


/*****************************************************************************/

// CNgramLearner

CNgramLearner::CNgramLearner()
{
}

/*****************************************************************************/

CNgramLearner::~CNgramLearner()
{
}

/*****************************************************************************/

void CNgramLearner::learn(CNgramNode *pmodel, int n, CString text)
/*
PURPOSE:      Learns by building an n-gram tree model with the given text for
				the given n value.
RECEIVES:   pmodel - The root node of the current n-gram model.
				n - The value of n of the n-gram i.e. n == 3 for a tri-gram.
				text - The text to build the n-gram model with.
*/
{
	int curTextPos = 0;		// the position of the word next segment should start with
	CList<CString> curSegment;
	CString curWord = text.Tokenize(CNgramWordPredictorDoc::DELIMITERS, curTextPos);
	if (curWord == "") return;	// No words in text

	// Grab the first segment of words
	curTextPos = 0;
	do
	{
		curWord = text.Tokenize(CNgramWordPredictorDoc::DELIMITERS, curTextPos);
		curSegment.AddTail(curWord.MakeLower());
	} while (curSegment.GetSize() < n && curWord != "");

	if (curWord == "")
	{	// Not enough words yet to build n-gram model
		return;
	}
	
	CNgramNode *pcurNode;
	CNgramNode *ptempNode;
	POSITION curPosInSeg;
	CString curSegWord;
	do
	{
		// Go down the n-gram model tree and increment the counter at the leaf
		curPosInSeg = curSegment.GetHeadPosition();
		pcurNode = pmodel;
		for (int i = 0; i < n; i++)
		{
			curSegWord = curSegment.GetNext(curPosInSeg);
			ptempNode = pcurNode->getChild(curSegWord);
			if (!ptempNode)
			{	// I don't have this node yet, so make it
				bool aa = curSegWord == "c";
				ptempNode = pcurNode->addChild(curSegWord);
			}
			pcurNode = ptempNode;
		}
		// Increment the counter at the leaf
		pcurNode->incCount();
		
		// Get the next segment and repeat
		curWord = text.Tokenize(CNgramWordPredictorDoc::DELIMITERS, curTextPos);
		curSegment.RemoveHead();
		curSegment.AddTail(curWord.MakeLower());
	} while (curWord != "");
}
