/******************************************************
* Copyright (c):   2004, All Rights Reserved.
* Project:         CS 297 Deliverable 1 N-gram Word Predictor
* File:            NgramNode.cpp
* Purpose:         A node in the n-gram tree.
* Start date:      8/30/2004
* Programmer:      Leo Lee
*
*******************************************************/

#include "stdafx.h"
#include "NgramWordPredictor.h"
#include "NgramNode.h"
#include ".\ngramnode.h"


/*****************************************************************************/

// CNgramNode

CNgramNode::CNgramNode(const CString &word, UINT count)
{
	m_word = word;
	m_count = count;
}

/*****************************************************************************/

CNgramNode::~CNgramNode()
{
	POSITION pos = m_children.GetHeadPosition();
	for (int i = 0; i < m_children.GetSize(); i++)
	{
		delete m_children.GetNext(pos);
	}
}

/*****************************************************************************/

CNgramNode *CNgramNode::getChild(const CString &word)
/*
PURPOSE:      Returns the n-gram node that is a child of this node with the given word.
RECEIVES:   word - The word that the desired child contains.
RETURNS:      The child node with the given word if it exists, else null.
*/
{
	POSITION pos = m_children.GetHeadPosition();
	CNgramNode *curNode;
	for (int i = 0; i < m_children.GetSize(); i++)
	{
		curNode = m_children.GetNext(pos);
		if (word == *curNode->getWord())
		{	// Found the word
			return curNode;
		}
	}
	return 0;	// no child node with this word
}

/*****************************************************************************/

CNgramNode *CNgramNode::addChild(const CString &word)
/*
PURPOSE:      Adds a child node to this node with the given word.
RECEIVES:   word - The word that the new child node will contain.
RETURNS:      Pointer to the new child node.
*/
{
	POSITION prevPos;
	POSITION pos = m_children.GetHeadPosition();
	for (int i = 0; i < m_children.GetSize(); i++)
	{
		prevPos = pos;
		if (word < *m_children.GetNext(pos)->getWord())
		{	// Found where word belongs
			if (prevPos == m_children.GetHeadPosition())
			{	// word belongs at head
				m_children.AddHead(new CNgramNode(word));
				return m_children.GetHead();
			} else
			{	// word belongs somewhere in the middle
				m_children.InsertBefore(prevPos, new CNgramNode(word));
				m_children.GetPrev(prevPos);
				return m_children.GetAt(prevPos);
			}
		}
	}
	// Reached the end of the list, so word must belong at the tail
	m_children.AddTail(new CNgramNode(word));
	return m_children.GetTail();
}

/*****************************************************************************/

void CNgramNode::Serialize(CArchive& ar)
/*
PURPOSE:      Save or load this node, and recursively, all of its children.
RECEIVES:   ar - stream to read from or write to.
*/

{
	int numChildren;
	POSITION pos;
	int size;
	if (ar.IsStoring())
	{	// storing code
		size = m_children.GetSize();
		ar << m_word << m_count << size;
		pos = m_children.GetHeadPosition();
		for (int i = 0; i < m_children.GetSize(); i++)
		{
			(m_children.GetNext(pos))->Serialize(ar);
		}
	}
	else
	{	// loading code
		m_children.RemoveAll();
		ar >> m_word >> m_count;
		ar >> numChildren;
		for (int i = 0; i < numChildren; i++)
		{
			CNgramNode *pnode = new CNgramNode("");
			pnode->Serialize(ar);
			m_children.AddTail(pnode);
		}
	}
}
