/******************************************************
* Copyright (c):   2004, All Rights Reserved.
* Project:         CS 297 Deliverable 1 N-gram Word Predictor
* File:            NgramPredictor.cpp
* Purpose:         Responsible for doing the prediction.
* Start date:      8/30/2004
* Programmer:      Leo Lee
*
*******************************************************/

#include "stdafx.h"
#include "NgramWordPredictor.h"
#include "NgramPredictor.h"
#include "NgramNode.h"
#include "NgramWordPredictorDoc.h"


/*****************************************************************************/

// CNgramPredictor

CNgramPredictor::CNgramPredictor()
{
}

/*****************************************************************************/

CNgramPredictor::~CNgramPredictor()
{
}

/*****************************************************************************/

CString CNgramPredictor::nextWord(CNgramNode *pmodel, int n, CString prevWords)
/*
PURPOSE:      Predicts the most likely next word based on the n-gram model
				and the previous n - 1 words in prevWords.
RECEIVES:   pmodel - The root node of the n-gram model.
				n - The n value of the n-gram e.g. n == 3 for a tri-gram
				prevWords - The previous entered text to base the prediction on.
RETURNS:      The predicted next work, or "" (empty string) if the next
				next word cannot be predicted i.e. prevWords does not contain
				at least n - 1 words.
*/
{
	// Get the last n - 1 words
	// First reverse all the text
	prevWords += " ";
	prevWords.MakeReverse();
	CString segment;
	CString curWord;
	int tokenPos = 0;
	// Then get the last n - 1 words that are reversed
	for (int i = 0; i < n - 1; i++)
	{
		curWord = prevWords.Tokenize(CNgramWordPredictorDoc::DELIMITERS, tokenPos);
		if (curWord == "") return "";		// Not enough words to predict: need at least n - 1 words
		segment += curWord + " ";
	}
	// Finally reverse the reversed words I got
	segment.MakeReverse();

	// Now we can traverse the n-gram tree and find the most probably next word
	// First let's get to the level right before the leaf nodes
	CNgramNode *curNode = pmodel;
	tokenPos = 0;
	for (int i = 0; i < n - 1; i++)
	{
		curNode = curNode->getChild(segment.Tokenize(CNgramWordPredictorDoc::DELIMITERS, tokenPos).MakeLower());
		if (curNode == 0) return "";
	}

	// Now we find the most likely next node from curNode's children i.e. highest count
	UINT highCount = 0;
	CNgramNode *predictNode = 0;
	CList<CNgramNode*> *pchildren = curNode->getChildren();
	POSITION curChildPos = pchildren->GetHeadPosition();
	for (int i = 0; i < pchildren->GetSize(); i++)
	{
		curNode = pchildren->GetNext(curChildPos);
		if (curNode->getCount() > highCount)
		{	// Found a more likely node
			highCount = curNode->getCount();
			predictNode = curNode;
		}
	}
	return *predictNode->getWord();
}