/******************************************************
* Copyright (c):   2004, All Rights Reserved.
* Project:         CS 297 Deliverable 1 N-gram Word Predictor
* File:            NgramWordPredictorDoc.cpp
* Purpose:         Doc class for the word predictor program
* Start date:      8/30/2004
* Programmer:      Leo Lee
*
*******************************************************/

#include "stdafx.h"
#include "NgramWordPredictor.h"
#include "NgramWordPredictorDoc.h"
#include "NgramLearner.h"
#include "NgramPredictor.h"
#include "NgramNode.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CNgramWordPredictorDoc

IMPLEMENT_DYNCREATE(CNgramWordPredictorDoc, CDocument)

BEGIN_MESSAGE_MAP(CNgramWordPredictorDoc, CDocument)
END_MESSAGE_MAP()


const char* CNgramWordPredictorDoc::DELIMITERS = " ,.?'\";:[]{}\\|+=-_)(*&^%$#@!~`<>";

/*****************************************************************************/

// CNgramWordPredictorDoc construction/destruction

CNgramWordPredictorDoc::CNgramWordPredictorDoc()
/*
PURPOSE:      Constructor, creates the learner, predictor and model to be used.
*/
{
	m_plearner = new CNgramLearner();
	m_ppredictor = new CNgramPredictor();
	m_pmodel = new CNgramNode("");
}

/*****************************************************************************/

CNgramWordPredictorDoc::~CNgramWordPredictorDoc()
/*
PURPOSE:      Destructor.
*/
{
	delete m_plearner;
	delete m_ppredictor;
	delete m_pmodel;
}

/*****************************************************************************/

BOOL CNgramWordPredictorDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	clearModel();
	// (SDI documents will reuse this document)

	return TRUE;
}

/*****************************************************************************/

// CNgramWordPredictorDoc serialization

void CNgramWordPredictorDoc::Serialize(CArchive& ar)
/*
PURPOSE:      Saves the n-gram model or loads model from a file.
RECEIVES:   The stream to read from or write to.
*/
{
	if (ar.IsStoring())
	{
		m_pmodel->Serialize(ar);
	}
	else
	{
		clearModel();
		m_pmodel = new CNgramNode("");
		m_pmodel->Serialize(ar);
	}
}

/*****************************************************************************/

// CNgramWordPredictorDoc diagnostics

#ifdef _DEBUG
void CNgramWordPredictorDoc::AssertValid() const
{
	CDocument::AssertValid();
}

/*****************************************************************************/

void CNgramWordPredictorDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


// CNgramWordPredictorDoc commands

void CNgramWordPredictorDoc::learn(CDialogNgram *pinputDialog)
/*
PURPOSE:      This function gets the n-value and text from the given input dialog,
				then invokes the learner to modify the n-gram model based on this
				new information.
RECEIVES:   pinputDialog - The input dialog to get the text from to learn.
*/
{
	pinputDialog->UpdateData(TRUE);
	CString text;
	pinputDialog->getTextArea()->GetWindowText(text);
	int n = pinputDialog->getNSelect()->GetCurSel() + 1;
	m_plearner->learn(m_pmodel, n, text);
	UpdateAllViews(0);
}

/*****************************************************************************/

CString CNgramWordPredictorDoc::predict(CDialogNgram *pinputDialog)
/*
PURPOSE:      Predicts the next word based on the previous n - 1 words 
				in the text area in pinputDialog.  
RECEIVES:   pinputDialog - The input dialog to get the text from to do the prediction.
RETURNS:      The predicted next word.
*/
{
	pinputDialog->UpdateData(TRUE);
	CString text;
	pinputDialog->getTextArea()->GetWindowText(text);
	int n = pinputDialog->getNSelect()->GetCurSel() + 1;
	return m_ppredictor->nextWord(m_pmodel, n, text);
}

/*****************************************************************************/

void CNgramWordPredictorDoc::clearModel()
/*
PURPOSE:      Removes all children nodes from the root n-gram model node;
				in effect, wiping out the current n-gram model.
*/
{
	delete m_pmodel;
	m_pmodel = new CNgramNode("");
	UpdateAllViews(0);
}
