/******************************************************
* Copyright (c):   2004, All Rights Reserved.
* Project:         CS 297 Deliverable 1 N-gram Word Predictor
* File:            NgramWordPredictorView.cpp
* Purpose:         View class for the word predictor program
* Start date:      8/30/2004
* Programmer:      Leo Lee
*
*******************************************************/

#include "stdafx.h"
#include "NgramWordPredictor.h"
#include "NgramWordPredictorDoc.h"
#include "NgramWordPredictorView.h"
#include "DialogNgram.h"
#include "NgramNode.h"
#include ".\ngramwordpredictorview.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CNgramWordPredictorView

IMPLEMENT_DYNCREATE(CNgramWordPredictorView, CView)

BEGIN_MESSAGE_MAP(CNgramWordPredictorView, CView)
	ON_WM_CREATE()
	ON_COMMAND(ID_VIEW_INPUTDIALOG, OnViewInputdialog)
END_MESSAGE_MAP()

// CNgramWordPredictorView construction/destruction

/*****************************************************************************/

CNgramWordPredictorView::CNgramWordPredictorView()
/*
PURPOSE:      Constructor for view class.  Set's up the input dialog.
*/
{
	((CNgramWordPredictorApp*)AfxGetApp())->setMainWindow(this);
	m_pinputDialog = new CDialogNgram(this);
}

/*****************************************************************************/

CNgramWordPredictorView::~CNgramWordPredictorView()
/*
PURPOSE:      Destructor.
*/
{
	delete m_pinputDialog;
}

/*****************************************************************************/

BOOL CNgramWordPredictorView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/*****************************************************************************/

// CNgramWordPredictorView drawing
static int yy = 0;
static int indent = 50;
static int offset = 20;

void CNgramWordPredictorView::OnDraw(CDC* pDC)
/*
PURPOSE:      Draws the display window i.e. the n-gram model, by delegating 
					task to displayModel().
RECEIVES:   pDC - pointer to device context for window.
*/
{
	CNgramWordPredictorDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	yy = 0;
	CArray<bool> drawVertBar;
	displayModel(pDC, pDoc->getModel(), 0, &drawVertBar);
}

/*****************************************************************************/

void CNgramWordPredictorView::displayModel(CDC *pDc, CNgramNode *pnode, int level, CArray<bool> *drawVertBar)
/*
PURPOSE:      Displays the n-gram model.
RECEIVES:   pDc - device context for window.
				pnode - current node in n-gram tree to display.
				level - how deep the current node is in the n-gram tree.
				drawVertBar - indicates whether a vertical bar should be drawn at certain parts of the display.
REMARKS:      The way the model is drawn is to indent each node's name by a certain offset
					dependent on the level of the node.  
*/
{
	CList<CNgramNode*>* pchildren = pnode->getChildren();
	POSITION curChildPos = pchildren->GetHeadPosition();
	// draw vertical bar
	for(int i = 0; i < level; i++)
	{
		if(drawVertBar->GetAt(i))
		{
			pDc->TextOut(i * indent + offset, yy, "|");
		}
	}
	// draw horizontal bar
	if(level > 0)
	{
		for(int i = 3; i < indent - 3; i++)
		{
			pDc->TextOut((level - 1) * indent + i + offset, yy, "-");
		}
	}

	char str[10] = "";
	if(pnode->getCount() != 0) 
	{
		sprintf(str, "(%d)", pnode->getCount());
	}
	pDc->TextOut(level * indent + offset, yy, str);
	pDc->TextOut(level * indent + offset + 25, yy, *pnode->getWord());

	yy += 20;
	for(int i = 0; i < pchildren->GetSize(); i++)
	{
		if(i < pchildren->GetSize() - 1)
		{
			drawVertBar->Add(true);
		} else
		{
			drawVertBar->Add(false);
		}
		displayModel(pDc, pchildren->GetNext(curChildPos), level + 1, drawVertBar);
		drawVertBar->RemoveAt(drawVertBar->GetSize() - 1);
	}
}

/*****************************************************************************/

// CNgramWordPredictorView diagnostics

#ifdef _DEBUG
void CNgramWordPredictorView::AssertValid() const
{
	CView::AssertValid();
}

/*****************************************************************************/

void CNgramWordPredictorView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

/*****************************************************************************/

CNgramWordPredictorDoc* CNgramWordPredictorView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CNgramWordPredictorDoc)));
	return (CNgramWordPredictorDoc*)m_pDocument;
}
#endif //_DEBUG

/*****************************************************************************/

// CNgramWordPredictorView message handlers

int CNgramWordPredictorView::OnCreate(LPCREATESTRUCT lpCreateStruct)
/*
PURPOSE:      Create the input dialog when the window is first created.
RECEIVES:   lpCreateStruct - only used in parent method.
REMARKS:      The input dialog cannot be created in the constructor because
				the window isn't create yet there.
*/
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_pinputDialog->Create(IDD_DIALOGNGRAM, this);
	m_pinputDialog->ShowWindow(SW_SHOW);

	EnableScrollBarCtrl(SB_VERT);
	EnableScrollBar(SB_VERT);
	SetScrollRange(SB_VERT, 0, 100);
	return 0;
}

/*****************************************************************************/

void CNgramWordPredictorView::OnViewInputdialog()
/*
PURPOSE:      Handler for menu selection, which displays the input dialog.
*/
{
	m_pinputDialog->ShowWindow(SW_SHOW);
}
